from ash import *

num_cores = 16   # running via slurm struggles here
                 # maybe pyscf will work??
#orcadir = "/home/software/ORCA.5.0.4/orca"

pdb_nam = "protein_and_ligand.pdb"
frag_prot = Fragment(pdbfile = pdb_nam)

# This 
thisdir = "/home/jzw5/quantum-olfaction-data/2024-05_protlig/2024-06_pyscf"

# For now, we're going to try only the ligand as active site.
lig_ind = [4836, 4837, 4838, 4839, 4840, 4841, 4842, 4843, 4844, 4845]
#         [C1, C2, C3, O2, O2, H21, H22, H31, H32, H33]
lig_mass = [12.01, 12.01, 12.01, 16.00, 16.00,
            2.014, 2.014, 2.014, 2.014, 2.014]

# MM region from OpenMM
# N.B.: I believe CHARMM36 actually means CHARMM36m, see
#   https://github.com/openmm/openmmforcefields/issues/298
# (CHARMM36m is an improved version, see
#   https://doi.org/10.1038/nmeth.4067)
# The GROMACS topology file is handwritten and in this directory, but it
# #includes two .itp files downloaded from 
# https://www.gpcrmd.org/dynadb/dynamics/id/1245/.
# 2024-05-20: Trying to use the ParMed library to read the topology file.
mm = OpenMMTheory(pdbfile = pdb_nam, 
                  GROMACSfiles = True,
                  grofile = "protein_and_ligand.gro",
                  gromacstopdir = thisdir,
                  gromacstopfile = "topol.top",
                  periodic = True,
                  platform = 'CPU',
                  numcores = num_cores,
                  autoconstraints = None,
                  rigidwater = False)

# Let's try QM from PySCF. Can it do parallelism?
qm = PySCFTheory(numcores = num_cores,
                 scf_type = 'RKS',
                 basis = 'aug-cc-pVTZ',
                 functional = 'b3lyp',
                 dispersion = 'D4')

# QM region from ORCA
# Functional: B3LYP (reasonable accuracy, efficient, analytic gradient)
#             with D4 dispersion correciton
# Basis:      minimally augmented Karlsruhe (doi:10.1007/s00214-010-0846-z)
#             we'll start with def2-TZVP, can *-TZVPP, *-QZVPP if needed
#             AutoAux: automatic RI from ORCA
#qm = ORCATheory(orcasimpleinput = "! B3LYP D4 ma-def2-TZVP AutoAux",
#                numcores = num_cores)

# QM/MM object combines
qmmm = QMMMTheory(qm_theory = qm, mm_theory = mm,
                  fragment = frag_prot,
                  qm_charge = -1,
                  #qm_mult = 1,
                  embedding = "Elstat",
                  qmatoms = lig_ind,
                  printlevel = 2)

# QM/MM geometry optimization only of the ligand!
active_ind = lig_ind
geom_opt = Optimizer(fragment = frag_prot, theory = qmmm,
                     ActiveRegion = True, actatoms = active_ind,
                     maxiter = 200,
                     charge = -1, mult = 1)

print("Optimized geometry! ", geom_opt)

# Vibrational frequency calculation.
# hessatoms = [active_ind] is a partial Hessian for just the active site!
# Use option hessatoms_masses = [...] to change the isotopes.
freq_num = NumFreq(fragment = frag_prot, theory = qmmm,
                   hessatoms = lig_ind,
                   hessatoms_masses = lig_mass,
                   numcores = 1, runmode = 'serial',
                   temp = 310)

print("Numerical frequency! ", freq_num)
print("hessian: ", freq_num.hessian)

print("Vibrational frequencies (cm**-1) : ", freq_num.frequencies)
print("IR intensities (km/mol)          : ", freq_num.IR_intensities)
